(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/common'), require('@angular/core'), require('rxjs')) :
    typeof define === 'function' && define.amd ? define('@studiohyperdrive/ngx-utils', ['exports', '@angular/common', '@angular/core', 'rxjs'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.studiohyperdrive = global.studiohyperdrive || {}, global.studiohyperdrive['ngx-utils'] = {}), global.ng.common, global.ng.core, global.rxjs));
}(this, (function (exports, i1, i0, rxjs) { 'use strict';

    // @dynamic
    var WindowService = /** @class */ (function () {
        function WindowService(document, 
        // tslint:disable-next-line: ban-types
        platformId) {
            var _this = this;
            this.document = document;
            this.platformId = platformId;
            this.widthSubject$ = new rxjs.BehaviorSubject(1200);
            this.scrollingUpSubject$ = new rxjs.BehaviorSubject(true);
            this.currentScrollPositionSubject$ = new rxjs.BehaviorSubject(0);
            /**
             * Observable to get the window-width, defaults to 1200 when no window is defined
             */
            this.width$ = this.widthSubject$.asObservable();
            /**
             * Observable to track when the scroll has ended
             */
            this.scrollingUp$ = this.scrollingUpSubject$.asObservable();
            /**
             * Observable of the current scroll position after the scroll has ended
             */
            this.currentScrollPosition$ = this.currentScrollPositionSubject$.asObservable();
            /**
             * Current scroll position after the scroll has ended
             */
            this.currentScrollPosition = 0;
            if (this.isBrowser() && this.hasDocument()) {
                this.window = this.document.defaultView;
                this.document.addEventListener('scroll', this.handleContentScroll.bind(this));
                this.widthSubject$.next(this.window.innerWidth);
                this.window.addEventListener('resize', function () {
                    if (_this.window.innerWidth && _this.widthSubject$.getValue()) {
                        _this.widthSubject$.next(_this.window.innerWidth);
                    }
                });
            }
        }
        /**
         * Scrolls to the provided position of the page
         *
         * @param offset - Offset to which we want to scroll, scrolls to top when no offset is provided
         */
        WindowService.prototype.scrollTo = function (offset) {
            if (offset === void 0) { offset = 0; }
            if (!this.window) {
                return;
            }
            this.window.scrollTo(0, offset);
        };
        /**
         * Returns whether there is a document present
         */
        WindowService.prototype.hasDocument = function () {
            return !!this.document;
        };
        /**
         * Returns whether the current platform is a browser
         */
        WindowService.prototype.isBrowser = function () {
            return i1.isPlatformBrowser(this.platformId);
        };
        WindowService.prototype.handleContentScroll = function () {
            if (window.pageYOffset > this.currentScrollPosition) {
                this.scrollingUpSubject$.next(false);
            }
            else {
                this.scrollingUpSubject$.next(true);
            }
            this.currentScrollPosition = window.pageYOffset < 0 ? 0 : window.pageYOffset;
            this.currentScrollPositionSubject$.next(this.currentScrollPosition);
        };
        return WindowService;
    }());
    WindowService.ɵprov = i0.ɵɵdefineInjectable({ factory: function WindowService_Factory() { return new WindowService(i0.ɵɵinject(i1.DOCUMENT), i0.ɵɵinject(i0.PLATFORM_ID)); }, token: WindowService, providedIn: "root" });
    WindowService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root',
                },] }
    ];
    WindowService.ctorParameters = function () { return [
        { type: Document, decorators: [{ type: i0.Inject, args: [i1.DOCUMENT,] }] },
        { type: String, decorators: [{ type: i0.Inject, args: [i0.PLATFORM_ID,] }] }
    ]; };

    /*
    * This windowMock constant return an object with a selected spy.
    * The reason why the spy is not provided by default is for this mock
    * to work on both Jest and Jasmine based test-suites.
    *
    * example:
    * windowMock(jasmine.createSpy());
    * */
    var windowMock = function (spy) { return ({
        addEventListener: spy,
        defaultView: {
            scrollTo: function () { return null; },
            innerWidth: 720,
            addEventListener: spy,
        },
    }); };
    /*
    * Provide a spy for your testing framework because the windowMock requires it.
    *
    * The reason why the spy is not provided by default is for this mock
    * to work on both Jest and Jasmine based test-suites.
    *
    * example:
    * windowServiceMock(jasmine.createSpy(), 1440);
    * */
    var windowServiceMock = function (spy, width) {
        if (width === void 0) { width = 1200; }
        return ({
            width: new rxjs.BehaviorSubject(width),
            window: windowMock(spy),
            scrollTo: function () { return null; },
            hasDocument: function () { return true; },
            isBrowser: function () { return true; },
        });
    };

    var SubscriptionService = /** @class */ (function () {
        function SubscriptionService() {
            this.destroyed$ = new rxjs.Subject();
        }
        SubscriptionService.prototype.ngOnDestroy = function () {
            this.destroyed$.next(true);
            this.destroyed$.complete();
        };
        return SubscriptionService;
    }());
    SubscriptionService.decorators = [
        { type: i0.Injectable }
    ];

    var UtilsModule = /** @class */ (function () {
        function UtilsModule() {
        }
        return UtilsModule;
    }());
    UtilsModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [],
                    imports: [],
                    providers: [WindowService],
                    exports: [],
                },] }
    ];

    /*
     * Public API Surface of utils
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.SubscriptionService = SubscriptionService;
    exports.UtilsModule = UtilsModule;
    exports.WindowService = WindowService;
    exports.windowMock = windowMock;
    exports.windowServiceMock = windowServiceMock;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=studiohyperdrive-ngx-utils.umd.js.map
