import { isPlatformBrowser, DOCUMENT } from '@angular/common';
import { ɵɵdefineInjectable, ɵɵinject, PLATFORM_ID, Injectable, Inject, NgModule } from '@angular/core';
import { BehaviorSubject, Subject } from 'rxjs';

// @dynamic
class WindowService {
    constructor(document, 
    // tslint:disable-next-line: ban-types
    platformId) {
        this.document = document;
        this.platformId = platformId;
        this.widthSubject$ = new BehaviorSubject(1200);
        this.scrollingUpSubject$ = new BehaviorSubject(true);
        this.currentScrollPositionSubject$ = new BehaviorSubject(0);
        /**
         * Observable to get the window-width, defaults to 1200 when no window is defined
         */
        this.width$ = this.widthSubject$.asObservable();
        /**
         * Observable to track when the scroll has ended
         */
        this.scrollingUp$ = this.scrollingUpSubject$.asObservable();
        /**
         * Observable of the current scroll position after the scroll has ended
         */
        this.currentScrollPosition$ = this.currentScrollPositionSubject$.asObservable();
        /**
         * Current scroll position after the scroll has ended
         */
        this.currentScrollPosition = 0;
        if (this.isBrowser() && this.hasDocument()) {
            this.window = this.document.defaultView;
            this.document.addEventListener('scroll', this.handleContentScroll.bind(this));
            this.widthSubject$.next(this.window.innerWidth);
            this.window.addEventListener('resize', () => {
                if (this.window.innerWidth && this.widthSubject$.getValue()) {
                    this.widthSubject$.next(this.window.innerWidth);
                }
            });
        }
    }
    /**
     * Scrolls to the provided position of the page
     *
     * @param offset - Offset to which we want to scroll, scrolls to top when no offset is provided
     */
    scrollTo(offset = 0) {
        if (!this.window) {
            return;
        }
        this.window.scrollTo(0, offset);
    }
    /**
     * Returns whether there is a document present
     */
    hasDocument() {
        return !!this.document;
    }
    /**
     * Returns whether the current platform is a browser
     */
    isBrowser() {
        return isPlatformBrowser(this.platformId);
    }
    handleContentScroll() {
        if (window.pageYOffset > this.currentScrollPosition) {
            this.scrollingUpSubject$.next(false);
        }
        else {
            this.scrollingUpSubject$.next(true);
        }
        this.currentScrollPosition = window.pageYOffset < 0 ? 0 : window.pageYOffset;
        this.currentScrollPositionSubject$.next(this.currentScrollPosition);
    }
}
WindowService.ɵprov = ɵɵdefineInjectable({ factory: function WindowService_Factory() { return new WindowService(ɵɵinject(DOCUMENT), ɵɵinject(PLATFORM_ID)); }, token: WindowService, providedIn: "root" });
WindowService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
WindowService.ctorParameters = () => [
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: String, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
];

/*
* This windowMock constant return an object with a selected spy.
* The reason why the spy is not provided by default is for this mock
* to work on both Jest and Jasmine based test-suites.
*
* example:
* windowMock(jasmine.createSpy());
* */
const windowMock = (spy) => ({
    addEventListener: spy,
    defaultView: {
        scrollTo: () => null,
        innerWidth: 720,
        addEventListener: spy,
    },
});
/*
* Provide a spy for your testing framework because the windowMock requires it.
*
* The reason why the spy is not provided by default is for this mock
* to work on both Jest and Jasmine based test-suites.
*
* example:
* windowServiceMock(jasmine.createSpy(), 1440);
* */
const windowServiceMock = (spy, width = 1200) => ({
    width: new BehaviorSubject(width),
    window: windowMock(spy),
    scrollTo: () => null,
    hasDocument: () => true,
    isBrowser: () => true,
});

class SubscriptionService {
    constructor() {
        this.destroyed$ = new Subject();
    }
    ngOnDestroy() {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    }
}
SubscriptionService.decorators = [
    { type: Injectable }
];

class UtilsModule {
}
UtilsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                providers: [WindowService],
                exports: [],
            },] }
];

/*
 * Public API Surface of utils
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SubscriptionService, UtilsModule, WindowService, windowMock, windowServiceMock };
//# sourceMappingURL=studiohyperdrive-ngx-utils.js.map
